@@ -10,6 +10,7 @@ error happened)::
         print 'height =', i.height
 
 """
+import types
 import numbers
 import collections
 import ctypes
@@ -18,7 +19,7 @@ import sys
 import warnings
 import platform
 from . import exceptions
-from .api import library
+from .api import library, libc
 from .resource import (increment_refcount, decrement_refcount, Resource,
                        DestroyedResourceError)
 from .color import Color
@@ -60,7 +61,11 @@ class Image(Resource):
 
     :param image: makes an exact copy of the ``image``
     :type image: :class:`Image`
-    :param filename: opens an image of the ``filename``
+    :param blob: opens an image of the ``blob`` byte array
+    :type blob: :class:`str`
+    :param file: opens an image of the ``file`` object
+    :type file: file object
+    :param filename: opens an image of the ``filename`` string
     :type filename: :class:`basestring`
 
     .. describe:: [left:right, top:bottom]
@@ -94,8 +99,8 @@ class Image(Resource):
 
     __slots__ = '_wand',
 
-    def __init__(self, image=None, blob=None, filename=None):
-        args = image, blob, filename
+    def __init__(self, image=None, blob=None, file=None, filename=None):
+        args = image, blob, file, filename
         if all(a is None for a in args):
             raise TypeError('missing arguments')
         elif any(a is not None and b is not None
@@ -111,6 +116,19 @@ class Image(Resource):
                 self.wand = library.CloneMagickWand(image.wand)
             else:
                 self.wand = library.NewMagickWand()
+                read = False
+                if file is not None:
+                    if isinstance(file, types.FileType):
+                        fd = libc.fdopen(file.fileno(), file.mode)
+                        library.MagickReadImageFile(self.wand, fd)
+                        read = True
+                    elif not callable(getattr(file, 'read', None)):
+                        raise TypeError('file must be a readable file object'
+                                        ', but the given object does not '
+                                        'have read() method')
+                    else:
+                        blob = file.read()
+                        file = None
                 if blob is not None:
                     if not isinstance(blob, collections.Iterable):
                         raise TypeError('blob must be iterable, not ' +
@@ -120,9 +138,11 @@ class Image(Resource):
                     elif not isinstance(blob, str):
                         blob = str(blob)
                     library.MagickReadImageBlob(self.wand, blob, len(blob))
+                    read = True
                 elif filename is not None:
                     library.MagickReadImage(self.wand, filename)
-                else:
+                    read = True
+                if not read:
                     raise TypeError('invalid argument(s)')
         self.raise_exception()
 
@@ -325,35 +345,59 @@ class Image(Resource):
         blur = ctypes.c_double(float(blur))
         library.MagickResizeImage(self.wand, width, height, filter, blur)
 
-    def save(self, filename):
-        """Saves the image into the ``filename``.
+    def save(self, file=None, filename=None):
+        """Saves the image into the ``file`` or ``filename``. It takes
+        only one argument at a time.
 
-        :param filename: a filename to write to
+        :param file: a file object to write to
+        :type file: file object
+        :param filename: a filename string to write to
         :type filename: :class:`basename`
 
         """
-        if not isinstance(filename, basestring):
-            raise TypeError('filename must be a string, not ' + repr(filename))
-        r = library.MagickWriteImage(self.wand, filename)
-        if not r:
-            self.raise_exception()
+        if file is None and filename is None:
+            raise TypeError('expected an argument')
+        elif file is not None and filename is not None:
+            raise TypeError('expected only one argument; but two passed')
+        elif file is not None:
+            if isinstance(file, types.FileType):
+                fd = libc.fdopen(file.fileno(), file.mode)
+                r = library.MagickWriteImageFile(self.wand, fd)
+                if not r:
+                    self.raise_exception()
+            else:
+                if not callable(getattr(file, 'write', None)):
+                    raise TypeError('file must be a writable file object, '
+                                    'but it does not have write() method: ' +
+                                    repr(file))
+                file.write(self.make_blob())
+        else:
+            if not isinstance(filename, basestring):
+                raise TypeError('filename must be a string, not ' +
+                                repr(filename))
+            r = library.MagickWriteImage(self.wand, filename)
+            if not r:
+                self.raise_exception()
 
-    def make_blob(self, format):
+    def make_blob(self, format=None):
         """Makes the binary string of the image.
 
-        :param format: the image format to write e.g. ``'png'``, ``'jpeg'``
+        :param format: the image format to write e.g. ``'png'``, ``'jpeg'``.
+                       it is omittable
         :type format: :class:`basestring`
         :returns: a blob (bytes) string
         :rtype: :class:`str`
         :raises: :exc:`ValueError` when ``format`` is invalid
 
         """
-        if not isinstance(format, basestring):
-            raise TypeError("format must be a string like 'png' or 'jpeg', "
-                            'not ' + repr(format))
-        r = library.MagickSetImageFormat(self.wand, str(format).strip().upper())
-        if not r:
-            raise ValueError('{0!r} is an invalid format'.format(format))
+        if format is not None:
+            if not isinstance(format, basestring):
+                raise TypeError("format must be a string like 'png' or 'jpeg'"
+                                ', not ' + repr(format))
+            r = library.MagickSetImageFormat(self.wand,
+                                             str(format).strip().upper())
+            if not r:
+                raise ValueError('{0!r} is an invalid format'.format(format))
         library.MagickResetIterator(self.wand)
         length = ctypes.c_size_t()
         blob_p = library.MagickGetImageBlob(self.wand, ctypes.byref(length))
